'use strict';
/* eslint-disable quote-props */

module.exports = {
    'plugins': ['stylelint-scss', 'stylelint-declaration-strict-value'],
    'extends': ['stylelint-config-recommended-scss'],
    'rules': {
        'at-rule-empty-line-before': [
            'always',
            {
                'except': [
                    'blockless-after-same-name-blockless',
                    'first-nested'
                ],
                'ignore': [
                    'after-comment'
                ],
                'ignoreAtRules': [
                    'else',
                    'if'
                ]
            }
        ],
        'at-rule-no-vendor-prefix': true,
        'block-closing-brace-newline-after': [
            'always',
            {
                'ignoreAtRules': [
                    'else',
                    'if'
                ]
            }
        ],
        'color-named': 'never',
        'comment-word-disallowed-list': [
            '/todo/i'
        ],
        'custom-property-pattern': [
            '^([\\#\\{\\$]*[a-z][a-z0-9\\}]*)(-[\\#\\$\\{a-z0-9\\}]+)*$',
            {
                message: 'Expected custom property name to be kebab-case'
            }
        ],
        'declaration-block-no-redundant-longhand-properties': null,
        'declaration-block-no-shorthand-property-overrides': null,
        'declaration-empty-line-before': [
            'always',
            {
                'except': [
                    'first-nested'
                ],
                'ignore': [
                    'after-comment',
                    'after-declaration'
                ]
            }
        ],
        'declaration-no-important': true,
        'function-comma-newline-after': null,
        'indentation': [
            4,
            {
                'indentClosingBrace': false,
                'ignore': [
                    'value'
                ]
            }
        ],
        'max-nesting-depth': [
            5,
            {
                'ignoreAtRules': [
                    'pseudo-classes',
                    'each',
                    'media',
                    'supports',
                    'include',
                    'at-root',
                    'if'
                ]
            }
        ],
        'max-line-length': [
            120,
            {
                'ignore': [
                    'comments'
                ]
            }
        ],
        'media-feature-name-no-vendor-prefix': true,
        'no-empty-first-line': true,
        'no-eol-whitespace': [
            true,
            {
                'ignore': [
                    'empty-lines'
                ]
            }
        ],
        'no-invalid-position-at-import-rule': null,
        'number-leading-zero': 'never',
        'property-no-vendor-prefix': true,
        'selector-max-compound-selectors': 4,
        'selector-max-id': 1,
        'selector-no-vendor-prefix': true,
        'selector-class-pattern': [
            '^([a-z][a-zA-Z0-9]*)([-_]*[a-zA-Z0-9]+)*$',
            {
                // eslint-disable-next-line max-len
                message: 'Selector can contain only hyphens or underscores for word separators (selector-name, selector-name__element)'
            }
        ],
        'selector-id-pattern': [
            '^([a-z][a-z0-9]*)([-_]?[a-z0-9]+)*$',
            {
                message: 'Id should be written in lowercase with hyphens or underscores (id-name, id_name)'
            }
        ],
        'keyframes-name-pattern': [
            '^([\\#\\{\\$]*[a-z][a-z0-9\\}]*)(-[\\#\\$\\{a-z0-9\\}]+)*$',
            {
                message: 'Expected keyframe name to be kebab-case'
            }
        ],
        'shorthand-property-no-redundant-values': true,
        'string-quotes': 'single',
        'value-no-vendor-prefix': [
            true,
            {
                'ignoreValues': [
                    'sticky'
                ]
            }
        ],
        'value-list-comma-newline-after': null,
        'scss/at-else-if-parentheses-space-before': 'always',
        'scss/at-else-empty-line-before': 'never',
        'scss/at-function-pattern': '^[a-z]+([a-z0-9-]+[a-z0-9]+)?$',
        'scss/at-function-parentheses-space-before': 'never',
        'scss/at-if-closing-brace-space-after': 'always-intermediate',
        'scss/at-mixin-pattern': '^[a-z]+([a-z0-9-]+[a-z0-9]+)?$',
        'scss/declaration-nested-properties': 'never',
        'scss/dollar-variable-colon-space-after': 'always-single-line',
        'scss/dollar-variable-colon-space-before': 'never',
        'scss/dollar-variable-pattern': '^[_]?[a-z]+([a-z0-9-]+[a-z0-9]+)?$',
        'scss/double-slash-comment-inline': 'never',
        'scss/selector-no-redundant-nesting-selector': true,
        // Disabled due to scss variable might have a long name
        'scss/operator-no-newline-after': null,
        // Disabled due to scss variable might have a long name
        'scss/operator-no-newline-before': null,
        'scale-unlimited/declaration-strict-value': [
            [
                'background-color',
                'border-color',
                'border-bottom-color',
                'border-left-color',
                'border-right-color',
                'border-top-color',
                'box-shadow',
                'color',
                'column-rule-color',
                'text-shadow',
                'outline-color'
            ],
            {
                'ignoreFunctions': false,
                'expandShorthand': true,
                'ignoreValues': [
                    'inset',
                    'unset',
                    'inherit',
                    'transparent',
                    'currentColor',
                    'initial',
                    'none',
                    'no-repeat',
                    'repeat-y',
                    'repeat-x',
                    'repeat',
                    'solid',
                    // ignore: 1px solid;
                    '/^\\d/',
                    // ignore: url(../../../img/loader-16.gif)
                    '/^(url.+)(\\.[gif|png|jpg.svg|webp]+)+/i',
                    // ignore: url($path);
                    '/\\$[_]?[a-z]+([a-z0-9-]+[a-z0-9]+)/i'
                ],
                // eslint-disable-next-line max-len
                'message': 'The "${value}" of "${property}" has to be a variable on declaration in scss $variable or css var(--variable)'
            }
        ]
    }
};
