const {transform, traverse, types: t, transformFromAst} = require('@babel/core');
const wrapperTemplate = require('./wrapper_template.js');

function processRequireCall(path) {
    const dependencyString = path.node.arguments[0].value;
    path.replaceWith(
        t.expressionStatement(
            t.conditionalExpression(
                t.callExpression(
                    t.memberExpression(t.identifier('__injections'), t.identifier('hasOwnProperty'), false),
                    [t.stringLiteral(dependencyString)]
                ),
                t.memberExpression(t.identifier('__injections'), t.stringLiteral(dependencyString), true),
                path.node
            )
        )
    );

    return dependencyString;
}

module.exports = function injectify(context, source, inputSourceMap) {
    const {ast} = transform(source, {
        babelrc: false,
        code: false,
        compact: false,
        ast: true,
        filename: context.resourcePath,
    });

    const dependencies = [];
    traverse(ast, {
        CallExpression(path) {
            if (t.isIdentifier(path.node.callee, {name: 'require'})) {
                dependencies.push(processRequireCall(path));
                path.skip();
            }
        },
    });

    if (dependencies.length === 0) {
        context.emitWarning(
            "The module you are trying to inject into doesn't have any dependencies. " +
            'Are you sure you want to do this?'
        );
    }

    const wrapperModuleAst = t.file(
        t.program([
            wrapperTemplate({
                SOURCE: t.blockStatement(ast.program.body),
                SOURCE_PATH: t.stringLiteral(context.resourcePath),
                DEPENDENCIES: t.arrayExpression(dependencies.map(d => t.stringLiteral(d))),
            }),
        ])
    );

    return transformFromAst(wrapperModuleAst, source, {
        sourceMaps: context.sourceMap,
        sourceFileName: context.resourcePath,
        inputSourceMap,
        babelrc: false,
        compact: false,
        filename: context.resourcePath,
    });
}
