const {once} = require('underscore');

class AfterWebpackLogsPlugin {
    /**
     * @param {function} afterLogsCb
     */
    constructor(afterLogsCb) {
        if (typeof afterLogsCb !== 'function') {
            throw new Error('The "afterLogsCb" arg should be a function');
        }
        this._afterLogsCb = once(afterLogsCb);
    }

    apply(compiler) {
        compiler.hooks.done.tap('AfterWebpackLogsPlugin', stats => {
            const compilerStats = stats;
            stats.compilation.hooks.statsPrinter.tap('AfterWebpackLogsPlugin', stats => {
                // Making logs to be after all webpack logs in the console
                setImmediate(() => this._afterLogsCb(compilerStats));
            });
        });
    }
}

module.exports = AfterWebpackLogsPlugin;
