module.exports = (resolver, config) => {
    return Object.entries(config)
        .map(([moduleName, shim]) => {
            let {imports, exports, expose} = shim;
            const uses = [];

            // convert to expose-loader? syntax
            if (expose) {
                expose = Array.isArray(expose) ? expose : [expose];
                uses.push(...expose.map(name => `expose-loader?exposes=${name}`));
            }

            // convert to imports-loader? syntax
            if (imports && imports.length) {
                let importParams = Object.assign(
                    {
                        type: 'commonjs',
                        imports: imports.filter(item => typeof item === 'string').join(',')
                    },
                    ...imports.filter(item => typeof item === 'object')
                );

                importParams = Object.entries(importParams)
                    .filter(([, value]) => Boolean(value))
                    .map(([key, value]) => `${key}=${value.replace(/ /g, '%20')}`);

                uses.push(`imports-loader?${importParams.join('&')}`);
            }

            // convert to exports-loader? syntax
            if (exports) {
                uses.push(`exports-loader?type=commonjs&exports=${exports}`);
            }

            return {
                test: resolver(moduleName),
                use: uses
            };
        });
};
