const BaseError = require('./base-error');

class AssetsInputFileError extends BaseError {
    /**
     * @example
     *  Invalid styles config: input file does not exist /scss/test.scss
     *  at /config/assets.yml
     *  Please, find more information in the documentation https://doc.oroinc.com/...
     *
     * @param {string} notExistFile
     * @param {string} path
     */
    constructor(notExistFile, path) {
        const msg = `input file does not exist: ${notExistFile}\nat ${path}:1`;

        super(msg);

        this.name = 'Invalid styles config';
        this.docLink =
            'https://doc.oroinc.com/backend/bundles/platform/AssetBundle/#load-scss-or-css-files-from-the-bundle';
    }
}

module.exports = AssetsInputFileError;
