const BaseError = require('./base-error');

class AssetsSchemaError extends BaseError {
    /**
     * @example
     *  Invalid styles config: the "assets.yml" in the "default" theme file does not match the API schema.
     *  styles.inputs[0] should be: string | object
     *  at /config/assets.yml
     *  Please, find more information in the documentation https://doc.oroinc.com/...
     *
     * @example
     * Invalid styles config: the "assets.yml" file the "default" theme does not match the API schema.
     * styles misses the property 'output'.
     * List of processed files for the "default" theme:
     *     /default/config/assets.yml
     *     /default/config/assets.yml
     *  Please, find more information in the documentation https://doc.oroinc.com/...
     *
     * @param {string} reason
     * @param {Array} files
     * @param {string} theme
     */
    constructor(reason, files, theme) {
        let msg = `the "assets.yml" file in the "${theme}" theme does not match the API schema.\n${reason}`;

        const filesListMsg = BaseError.generateFilesListMsg(files, theme);

        if (filesListMsg.length) {
            msg = `${msg}\n${filesListMsg}`;
        }

        super(msg);

        this.name = 'Invalid styles config';
        this.docLink =
            'https://doc.oroinc.com/backend/bundles/platform/AssetBundle/#load-scss-or-css-files-from-the-bundle';
    }
}

module.exports = AssetsSchemaError;
