const {isVerboseMode} = require('../../utils');

class BaseError extends Error {
    /**
     * Prepares documentation phrase
     * @returns {string}
     */
    get extra() {
        return this.docLink ? `Please, find more information in the documentation ${this.docLink}` : '';
    }

    /**
     * Creates an instance of BaseError
     * @param {string} message error message
     */
    constructor(message) {
        super(message);
    }

    /**
     * Creates a part of message about processed files
     * @param {Array} files
     * @param {string} theme
     * @returns {string}
     */
    static generateFilesListMsg(files, theme) {
        if (files.length === 1) {
            return `at ${files[0]}:1`;
        } else if (isVerboseMode()) {
            return `List of processed files for the "${theme}" theme:\n    ${files.join('\n    ')}`;
        }

        return '';
    }
}

module.exports = BaseError;
