const BaseError = require('./base-error');

class JSModulesSchemaError extends BaseError {
    /**
     * @example
     * Invalid JS config: the "jsmodules.yml" files in the "default" theme do not match the API schema.
     * has an unknown property 'rest'.
     * at default/config/jsmodules.yml
     * Please, find more information in the documentation https://doc.oroinc.com/...
     *
     * @example
     * Invalid JS config: the "jsmodules.yml" files in the "default" theme do not match the API schema.
     * misses the property 'entry'.
     * List of processed files for the "default" theme:
     *     /default/config/jsmodules.yml
     *     /default/config/jsmodules.yml
     * Please, find more information in the documentation https://doc.oroinc.com/...
     *
     * @param {string} reason
     * @param {Array} files
     * @param {string} theme
     */
    constructor(reason, files, theme) {
        let msg = `the "jsmodules.yml" files in the "${theme}" theme do not match the API schema.\n${reason}`;

        const filesListMsg = BaseError.generateFilesListMsg(files, theme);

        if (filesListMsg.length) {
            msg = `${msg}\n${filesListMsg}`;
        }

        super(msg);

        this.name = 'Invalid JS config';
        this.docLink =
            'https://doc.oroinc.com/backend/bundles/platform/AssetBundle/#create-jsmodules-yml-configuration';
    }
}

module.exports = JSModulesSchemaError;
