const BaseError = require('./base-error');

class StylesError extends BaseError {
    /**
     * @example
     * Failed assembly styles: the "output" for "styles" entry point in "default" theme is not defined.
     * Please, find more information in the documentation https://doc.oroinc.com/...
     *
     * @param {string} key
     * @param {string} group
     * @param {string} theme
     */
    constructor(key, group, theme) {
        const msg = `the "${key}" for "${group}" entry point in "${theme}" theme is not defined.`;

        super(msg);

        this.name = 'Failed assembly styles';
        this.docLink =
            'https://doc.oroinc.com/backend/bundles/platform/AssetBundle/#load-scss-or-css-files-from-the-bundle';
    }
}

module.exports = StylesError;
