const assetsValidation = require('./assets-validator');
const jsmodulesValidator = require('./jsmodules-validator');

const isJSModulesPath = path => /jsmodules([-a-zA-Z\d]*)\.yml$/.test(path);
const isAssetsPath = path => /assets\.yml$/.test(path);

module.exports = {
    assetsValidation,
    jsmodulesValidator,

    /**
     * set public path for all validators
     * {string} path
     */
    setPublicPath(path) {
        [assetsValidation, jsmodulesValidator].forEach(validator => validator.setPublicPath(path));
    },

    /**
     * Run appropriate validator
     * {string} filePath
     * {Object} doc
     * {string} theme
     */
    checkSchema(filePath, doc, theme) {
        if (isJSModulesPath(filePath)) {
            jsmodulesValidator.checkSchema(filePath, doc, theme);
        } else if (isAssetsPath(filePath)) {
            const validSchema = assetsValidation.checkSchema(filePath, doc, theme);

            if (validSchema) {
                assetsValidation.checkInputsExist(filePath, doc, theme);
            }
        }
    }
};
