const schema = require('./schemas/jsmodules-schema');
const fullSchema = require('./schemas/jsmodules-schema-full');
const schemaValidator = require('./schema-validator');
const {isProdMode} = require('../utils');
const EventEmitter = require('events');
const emitter = new EventEmitter();
const JSModulesSchemaError = require('./errors/jsmodules-schema-error');

module.exports = Object.assign({}, schemaValidator, {
    emitter,

    /**
     * @param {string} filePath
     * @param {Object} doc
     * @param {string} theme
     * @returns {boolean|undefined}
     */
    checkSchema(filePath, doc, theme) {
        if (isProdMode()) {
            return;
        }
        const result = this.validateSchema(schema, doc);

        if (!result.valid) {
            const error = new JSModulesSchemaError(result.formattedError, [filePath], theme);

            this.emitter.emit('error', error);
        }

        return result.valid;
    },

    /**
     * @param {Object} doc
     * @param {Array} files
     * @param {string} theme
     * @returns {boolean|undefined}
     */
    checkFullSchema(doc, files = [], theme) {
        if (isProdMode()) {
            return;
        }
        const result = this.validateSchema(fullSchema, doc);

        if (!result.valid) {
            const error = new JSModulesSchemaError(result.formattedError, files, theme);

            this.emitter.emit('error', error);
        }

        return result.valid;
    }
});
