const {validate} = require('schema-utils');

module.exports = {
    /**
     * Symfony public directory path related to application root folder
     * {string}
     */
    _publicPath: 'public/',

    setPublicPath(path) {
        this._publicPath = path;
    },

    /**
     * Validates data according to the schema
     * @param {Object} scheme
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Object}
     */
    validateSchema(scheme, data, options) {
        const result = {
            valid: true
        };

        try {
            const baseDataPath = 'configuration';
            const basePathReg = /^configuration\./;
            validate(scheme, data, {
                baseDataPath,
                ...options || {},
                postFormatter(formattedError, error) {
                    if (result.formattedError === void 0) {
                        result.formattedError = [];
                    }
                    result.error = error;

                    if (
                        Array.isArray(error.params.type) ||
                        error.params.missingProperty
                    ) {
                        // remove an excess line break
                        formattedError = formattedError.replace(/\n/g, ' ');
                    }

                    formattedError = formattedError.replace(basePathReg, '');
                    result.formattedError.push(formattedError.trim());
                    return formattedError;
                }
            });
        } catch (e) {
            result.valid = false;
            result.errorMessage = e.message;
        }

        if (result.formattedError) {
            result.formattedError = result.formattedError.join('\n');
        }

        return result;
    }
};
